package javax.swing.plaf.metal;

import java.beans.PropertyChangeEvent;
import javax.swing.tree.TreePath;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.Insets;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import java.awt.Color;
import javax.swing.plaf.basic.BasicTreeUI;

public class MetalTreeUI extends BasicTreeUI
{
    private static Color lineColor;
    private static final String LINE_STYLE = "JTree.lineStyle";
    private static final String LEG_LINE_STYLE_STRING = "Angled";
    private static final String HORIZ_STYLE_STRING = "Horizontal";
    private static final String NO_STYLE_STRING = "None";
    private static final int LEG_LINE_STYLE = 2;
    private static final int HORIZ_LINE_STYLE = 1;
    private static final int NO_LINE_STYLE = 0;
    private int lineStyle;
    private PropertyChangeListener lineStyleListener;
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalTreeUI();
    }
    
    public MetalTreeUI() {
        this.lineStyle = 2;
        this.lineStyleListener = new MetalTreeUI.LineListener();
    }
    
    @Override
    protected int getHorizontalLegBuffer() {
        return 3;
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        MetalTreeUI.lineColor = UIManager.getColor("Tree.line");
        this.decodeLineStyle(c.getClientProperty("JTree.lineStyle"));
        c.addPropertyChangeListener(this.lineStyleListener);
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        c.removePropertyChangeListener(this.lineStyleListener);
        super.uninstallUI(c);
    }
    
    protected void decodeLineStyle(Object o) {
        if (o == null || o.equals("Angled")) {
            this.lineStyle = 2;
        }
        else if (o.equals("None")) {
            this.lineStyle = 0;
        }
        else if (o.equals("Horizontal")) {
            this.lineStyle = 1;
        }
    }
    
    protected boolean isLocationInExpandControl(int row, int n, int n2, int n3) {
        if (this.tree != null && !this.isLeaf(row)) {
            int n4;
            if (this.getExpandedIcon() != null) {
                n4 = this.getExpandedIcon().getIconWidth() + 6;
            }
            else {
                n4 = 8;
            }
            Insets insets = this.tree.getInsets();
            int n5 = ((insets != null) ? insets.left : 0) + ((n + this.depthOffset - 1) * this.totalChildIndent + this.getLeftChildIndent() - n4 / 2);
            int n6 = n5 + n4;
            return n2 >= n5 && n2 <= n6;
        }
        else {
            return false;
        }
    }
    
    @Override
    public void paint(Graphics g, JComponent c) {
        super.paint(g, c);
        if (this.lineStyle == 1 && !this.largeModel) {
            this.paintHorizontalSeparators(g, c);
        }
    }
    
    protected void paintHorizontalSeparators(Graphics graphics, JComponent component) {
        graphics.setColor(MetalTreeUI.lineColor);
        Rectangle clipBounds = graphics.getClipBounds();
        int i = this.getRowForPath(this.tree, this.getClosestPathForLocation(this.tree, 0, clipBounds.y));
        int rowForPath = this.getRowForPath(this.tree, this.getClosestPathForLocation(this.tree, 0, clipBounds.y + clipBounds.height - 1));
        if (i <= -1 || rowForPath <= -1) {
            return;
        }
        for (; i <= rowForPath; ++i) {
            TreePath pathForRow = this.getPathForRow(this.tree, i);
            if (pathForRow != null && pathForRow.getPathCount() == 2) {
                Rectangle pathBounds = this.getPathBounds(this.tree, this.getPathForRow(this.tree, i));
                if (pathBounds != null) {
                    graphics.drawLine(clipBounds.x, pathBounds.y, clipBounds.x + clipBounds.width, pathBounds.y);
                }
            }
        }
        return;
    }
    
    @Override
    protected void paintVerticalPartOfLeg(Graphics g, Rectangle clipBounds, Insets insets, TreePath path) {
        if (this.lineStyle == 2) {
            super.paintVerticalPartOfLeg(g, clipBounds, insets, path);
        }
    }
    
    @Override
    protected void paintHorizontalPartOfLeg(Graphics g, Rectangle clipBounds, Insets insets, Rectangle bounds, TreePath path, int row, boolean isExpanded, boolean hasBeenExpanded, boolean isLeaf) {
        if (this.lineStyle == 2) {
            super.paintHorizontalPartOfLeg(g, clipBounds, insets, bounds, path, row, isExpanded, hasBeenExpanded, isLeaf);
        }
    }
    
    class LineListener implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (propertyChangeEvent.getPropertyName().equals("JTree.lineStyle")) {
                MetalTreeUI.this.decodeLineStyle(propertyChangeEvent.getNewValue());
            }
        }
    }
}
